﻿using Contracts.BindingModels;
using Contracts.ViewModels;
using DataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace DatabaseImplement.Models
{
	public class Product : IProductModel
	{
		public int Id { get; set; }
		[Required]
		public string Name { get; set; } = string.Empty;
		[Required]
		public double Cost { get; set; }
		[Required]
		public int UserId { get; set; }
		public virtual Implementer User { get; set; }
		private Dictionary<int, (IDetailModel, int)>? _detailProducts = null;
		[NotMapped]
		public Dictionary<int, (IDetailModel, int)> DetailProducts
		{
			get
			{
				if (_detailProducts == null)
				{
					_detailProducts = Details.ToDictionary(recDP => recDP.DetailId, recDP => (recDP.Detail as IDetailModel, recDP.Count));
				}
				return _detailProducts;
			}
		}

		[ForeignKey("ProductId")]
		public virtual List<DetailProduct> Details { get; set; } = new();
		public static Product? Create(FactoryGoWorkDatabase context, ProductBindingModel model)
		{
			if (model == null)
			{
				return null;
			}
			return new Product
			{
				Id = model.Id,
				Name = model.Name,
				Cost = model.Cost,
				UserId = model.UserId,
				Details = model.ProductDetails.Select(x => new DetailProduct
				{
					Detail = context.Details.First(y => y.Id == x.Key),
					Count = x.Value.Item2
				}).ToList()
			};
		}
		public static Product Create(ProductViewModel model)
		{
			return new Product
			{
				Id = model.Id,
				Name = model.Name,
				Cost = model.Cost,
				UserId = model.UserId
			};
		}
		public void Update(ProductBindingModel model)
		{
			if (model == null)
				return;
			Name = model.Name;
			Cost = model.Cost;
		}
		public ProductViewModel GetViewModel => new()
		{
			Id = Id,
			Name = Name,
			Cost = Cost,
			UserId = UserId,
			DetailProducts = DetailProducts
		};
		public void UpdateDetails(FactoryGoWorkDatabase context, ProductBindingModel model)
		{
			var productDetails = context.DetailProducts.Where(rec => rec.ProductId == model.Id).ToList();
			if (productDetails != null && productDetails.Count > 0)
			{
				context.DetailProducts.RemoveRange(productDetails.Where(rec => !model.ProductDetails.ContainsKey(rec.DetailId)));
				context.SaveChanges();
				foreach (var upDetail in productDetails)
				{
					upDetail.Count = model.ProductDetails[upDetail.DetailId].Item2;
					model.ProductDetails.Remove(upDetail.DetailId);
				}
				context.SaveChanges();
			}
			var product = context.Products.First(x => x.Id == model.Id);
			foreach (var dp in model.ProductDetails)
			{
				context.DetailProducts.Add(new DetailProduct
				{
					Product = product,
					Detail = context.Details.First(x => x.Id == dp.Key),
					Count = dp.Value.Item2
				});
				context.SaveChanges();
			}
			_detailProducts = null;
		}
	}
}
